from zemfrog.decorators import http_code, authenticate
from zemfrog.helper import db_add, db_delete, db_update
from zemfrog.models import DefaultResponseSchema
from flask_apispec import marshal_with, use_kwargs
from marshmallow import fields
from zemfrog.globals import ma
from {{ "" if main_app else ".." }}{{src_model}} import {{name}}

class Create{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}
        exclude = ("id",)

class Read{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}

class Update{{name}}Schema(ma.SQLAlchemyAutoSchema):
    class Meta:
        model = {{name}}
        exclude = ("id",)

# class Delete{{name}}Schema(ma.SQLAlchemyAutoSchema):
#     class Meta:
#         model = {{name}}

class Limit{{name}}Schema(ma.Schema):
    offset = fields.Integer()
    limit = fields.Integer()


@authenticate()
@use_kwargs(Limit{{name}}Schema(), location="query")
@marshal_with(Read{{name}}Schema(many=True), 200)
def read(**kwds):
    """
    Read all data.
    """

    offset = kwds.get("offset")
    limit = kwds.get("limit")
    data = {{name}}.query.offset(offset).limit(limit).all()
    return data

@authenticate()
@use_kwargs(Create{{name}}Schema())
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 403)
@http_code
def create(**kwds):
    """
    Add data.
    """

    found = {{name}}.query.filter_by(**kwds).first()
    if not found:
        model = {{name}}(**kwds)
        db_add(model)
        status_code = 200
        message = "Successfully added data."

    else:
        status_code = 403
        message = "Data already exists."

    return {
        "code": status_code,
        "message": message
    }

@authenticate()
@use_kwargs(Update{{name}}Schema())
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 404)
@http_code
def update(id, **kwds):
    """
    Update data.
    """

    model = {{name}}.query.get(id)
    if model:
        db_update(model, **kwds)
        status_code = 200
        message = "Successfully updating data."

    else:
        status_code = 404
        message = "Data not found."

    return {
        "code": status_code,
        "message": message
    }

@authenticate()
# @use_kwargs(Delete{{name}}Schema())
@marshal_with(DefaultResponseSchema, 200)
@marshal_with(DefaultResponseSchema, 404)
@http_code
def delete(id):
    """
    Delete data.
    """

    model = {{name}}.query.get(id)
    if model:
        db_delete(model)
        status_code = 200
        message = "Data deleted successfully."

    else:
        status_code = 404
        message = "Data not found."

    return {
        "code": status_code,
        "message": message
    }


docs = {"tags": ["{{name}}"]}
endpoint = "{{url_prefix | lower}}"
url_prefix = "/{{url_prefix | lower}}"
routes = [
    ("/create", create, ["POST"]),
    ("/read", read, ["GET"]),
    ("/update/<id>", update, ["PUT"]),
    ("/delete/<id>", delete, ["DELETE"])
]



import re

from marshmallow.exceptions import ValidationError
from marshmallow.validate import Email


def validate_module_name(value: str, silently=False):
    re_var = re.compile(r"^([A-Za-z_]+[\w]+)$")
    if not re_var.search(value):
        if silently:
            return False
        raise ValidationError("Error: invalid name %r" % value)
    return True


def validate_email(value: str, silently=False):
    try:
        Email()(value)
    except ValidationError as e:
        if silently:
            return False
        raise ValidationError(e.messages[0])
    return True


def validate_username(value: str, silently=False):
    if not re.search(r"^([a-zA-Z]+)$", value):
        if silently:
            return False
        raise ValidationError("Name must be a character [a-zA-Z]")
    return True


def validate_password_length(value: str, silently=False):
    if len(value) < 8:
        if silently:
            return False
        raise ValidationError("Password length must be greater than or equal to 8")
    return True



from marshmallow import Schema, ValidationError, fields, validates

from .validators import validate_password_length, validate_username


class DefaultResponseSchema(Schema):
    message = fields.String()
    code = fields.Integer()


class LoginSuccessSchema(Schema):
    access_token = fields.String()


class LoginSchema(Schema):
    username = fields.Email(required=True)
    password = fields.String(required=True)
    grant_type = fields.String(default="password")


class RegisterSchema(Schema):
    first_name = fields.String(required=True)
    last_name = fields.String(required=True)
    username = fields.Email(required=True)
    password = fields.String(required=True)

    @validates("first_name")
    def validate_first_name(self, value):
        if not validate_username(value, silently=True):
            raise ValidationError("First name must be a character [a-zA-Z]")

    @validates("last_name")
    def validate_last_name(self, value):
        if not validate_username(value, silently=True):
            raise ValidationError("Last name must be a character [a-zA-Z]")

    @validates("password")
    def validate_password(self, value):
        if not validate_password_length(value, silently=True):
            raise ValidationError("Password length must be greater than or equal to 8")


class RequestPasswordResetSchema(Schema):
    username = fields.Email(required=True)


class PasswordResetSchema(Schema):
    password = fields.String(required=True)

    @validates("password")
    def validate_password(self, value):
        if not validate_password_length(value, silently=True):
            raise ValidationError("Password length must be greater than or equal to 8")



import pkg_resources
from flask import Flask
from flask.cli import routes_command, run_command, shell_command

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    A function to load all your commands and register them in the ``flask`` command.
    """

    dirname = "commands"
    commands = app.config.get(dirname.upper(), [])
    import_name = get_import_name(app)
    prefix = dirname + "."
    for name in commands:
        if not name.startswith(prefix):
            name = prefix + name

        name += ".command"
        try:
            cmd = import_attr(import_name + name)
        except ImportError:
            cmd = import_attr(name.lstrip(prefix))

        app.cli.add_command(cmd)

    if import_name:
        for cmd in (run_command, shell_command, routes_command):
            app.cli.add_command(cmd)

        for ep in pkg_resources.iter_entry_points("flask.commands"):
            app.cli.add_command(ep.load(), ep.name)



import os

import click


@click.group("secretkey")
def group():
    """
    Secret key generator.
    """


@group.command()
@click.argument("length", type=int)
def new(length):
    """
    Generate a random secret key.
    """

    print(os.urandom(length).hex())


command = group



from . import views

docs = {"tags": ["auth"]}
routes = [
    ("/user", views.user_detail, ["GET"]),
    ("/login", views.login, ["POST"]),
    ("/register", views.register, ["POST"]),
    ("/confirm/<token>", views.confirm_account, ["GET"]),
    ("/forgot-password", views.request_password_reset, ["POST"]),
    ("/password-reset/verify/<token>", views.confirm_password_reset_token, ["GET"]),
    ("/password-reset/<token>", views.password_reset, ["POST"]),
]



from flask import jsonify


def get():
    return jsonify(text="welcome :')")


docs = {"tags": ["{{name}}"]}
endpoint = "{{url_prefix | lower}}"
url_prefix = "/{{url_prefix | lower}}"
routes = [("/get", get, ["GET"])]



import click
from flask.cli import with_appcontext

from ..helper import db_add, db_commit, db_delete, db_update, get_object_model


@click.group("role")
def group():
    """
    Access role manager.
    """


@group.command()
@with_appcontext
@click.argument("name")
@click.option("-d", "--description", required=True, help="Role description.")
@click.option("-p", "--permissions", help="Role permissions.")
def new(name, description, permissions):
    """
    Create role.
    """

    model = get_object_model("role")
    perm_model = get_object_model("permission")
    role = model.query.filter_by(name=name).first()
    if not role:
        role = model(name=name, description=description)
        if permissions:
            for perm_name in permissions.split(","):
                perm = perm_model.query.filter_by(name=perm_name).first()
                if perm:
                    success = role.add_perm(perm)
                    if success:
                        msg = "Successfully added %r permission" % perm_name
                    else:
                        msg = "%r permission already exists" % perm_name
                else:
                    msg = "%r permission does not exist" % perm_name

                print(msg)

        db_add(role)
        print("Successfully added %r role" % name)
    else:
        print("%r role already exists" % name)


@group.command()
@with_appcontext
@click.argument("name")
def remove(name):
    """
    Remove role.
    """

    model = get_object_model("role")
    role = model.query.filter_by(name=name).first()
    if role:
        db_delete(role)
        print("Successfully deleted %r role" % name)
    else:
        print("%r role doesn't exist" % name)


@group.command()
@with_appcontext
@click.argument("name")
def update(name):
    """
    Update role.
    """

    model = get_object_model("role")
    role = model.query.filter_by(name=name).first()
    if role:
        cols = {}
        new_name = input("Enter a new role name: ").strip()
        description = input("Enter a new role description: ").strip()
        permissions = input("Enter new permissions: ").strip()
        if not new_name:
            new_name = role.name
        cols["name"] = new_name

        if not description:
            description = role.description
        cols["description"] = description

        if permissions:
            perm_model = get_object_model("permission")
            for perm_name in permissions.split(","):
                perm = perm_model.query.filter_by(name=perm_name).first()
                if perm:
                    success = role.add_perm(perm)
                    if success:
                        msg = "Successfully added %r permission" % perm_name
                    else:
                        msg = "%r permission already exists" % perm_name
                else:
                    msg = "%r permission does not exist" % perm_name

                print(msg)

        db_update(role, **cols)
        print("Successfully updated the %r role to %r" % (name, new_name))
    else:
        print("%r role doesn't exist" % name)


@group.command()
@with_appcontext
def list():
    """
    Show roles.
    """

    model = get_object_model("role")
    roles = model.query.all()
    print("Total roles: %d" % len(roles))
    for role in roles:
        perms = [perm.name for perm in role.permissions]
        print("* %s - %s (%s)" % (role.name, role.description, ", ".join(perms)))


@group.command()
@with_appcontext
def drop():
    """
    Drop roles.
    """

    print("Dropping all roles... ", end="")
    model = get_object_model("role")
    model.query.delete()
    db_commit()
    print("(done)")


command = group



import os
from glob import glob
from importlib import import_module

from flask import Flask

from ..helper import get_import_name, get_models


def loader(app: Flask):
    """
    A function to load all your ORM models in the ``models`` directory.
    """

    app.models = {}
    true = app.config.get("CREATE_DB")
    import_name = get_import_name(app)
    if import_name:
        import_name = import_name.replace(".", "/")

    if true:
        models = [
            x.rsplit(".", 1)[0].replace(os.sep, ".")
            for x in glob(import_name + "models/**/*.py", recursive=True)
        ]
        for m in models:
            if "__init__" in m:
                m = m.replace(".__init__", "")
            mod = import_module(m)
            app.models[m] = get_models(mod)

        app.extensions["sqlalchemy"].db.create_all()



from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import relationship
from sqlalchemy_utils import EmailType

from .exception import ZemfrogRoleNotFound, ZemfrogRolePermissionNotFound
from .helper import db_commit, db_update


class UserMixin:
    """
    User model.
    """

    id = Column(Integer, primary_key=True)
    first_name = Column(String(255), nullable=False)
    last_name = Column(String(255), nullable=False)
    name = Column(String(255), nullable=False)
    email = Column(EmailType, nullable=False, unique=True)
    password = Column(String(255), nullable=False)
    register_at = Column(DateTime)
    confirmed = Column(Boolean)
    confirmed_at = Column(DateTime)

    @declared_attr
    def logs(cls):
        return relationship("Log")

    @declared_attr
    def roles(cls):
        return relationship("Role")

    def get_role(self, name, silently=False):
        for role in self.roles:
            if role.name == name:
                return role

        if silently:
            return False

        raise ZemfrogRoleNotFound

    def has_role(self, name):
        try:
            self.get_role(name)
        except ZemfrogRoleNotFound:
            return False
        else:
            return True

    def add_role(self, role):
        have = self.get_role(role.name, silently=True)
        if not have:
            self.roles.append(role)
            db_commit()
            return True
        return False

    def delete_role(self, name):
        role = self.get_role(name, silently=True)
        if role:
            self.roles.remove(role)
            db_commit()
            return True
        return False

    def update_role(self, name, **kwds):
        role = self.get_role(name, silently=True)
        if role:
            db_update(role, **kwds)
            return True
        return False


class RoleMixin:
    """
    Role user.
    """

    id = Column(Integer, primary_key=True)

    @declared_attr
    def user_id(cls):
        return Column(ForeignKey("user.id"))

    name = Column(String(255), nullable=False, unique=True)
    description = Column(String(255), nullable=False)

    @declared_attr
    def permissions(cls):
        return relationship("Permission")

    def get_perm(self, name, silently=False):
        for perm in self.permissions:
            if perm.name == name:
                return perm

        if silently:
            return False

        raise ZemfrogRolePermissionNotFound

    def has_perm(self, name):
        try:
            self.get_perm(name)
        except ZemfrogRolePermissionNotFound:
            return False
        else:
            return True

    def add_perm(self, perm):
        have = self.get_perm(perm.name, silently=True)
        if not have:
            self.permissions.append(perm)
            db_commit()
            return True
        return False

    def delete_perm(self, name):
        perm = self.get_perm(name, silently=True)
        if perm:
            self.permissions.remove(perm)
            db_commit()
            return True
        return False

    def update_perm(self, name, **kwds):
        perm = self.get_perm(name, silently=True)
        if perm:
            db_update(perm, **kwds)
            return True
        return False


class PermissionMixin:
    """
    Role permission.
    """

    id = Column(Integer, primary_key=True)

    @declared_attr
    def role_id(cls):
        return Column(ForeignKey("role.id"))

    name = Column(String(255), nullable=False, unique=True)
    description = Column(String(255), nullable=False)


class LogMixin:
    """
    User activity log.
    """

    id = Column(Integer, primary_key=True)

    @declared_attr
    def user_id(cls):
        return Column(ForeignKey("user.id"))

    login_at = Column(DateTime)
    request_password_reset_at = Column(DateTime)
    updated_at = Column(DateTime)



from flask import Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    A function to load all jinja filters into a flask.
    """

    filters = app.config.get("JINJA_FILTERS", [])
    import_name = get_import_name(app)
    prefix = "filters."
    for name in filters:
        if not name.startswith(prefix):
            name = prefix + name

        name += ".init_filter"

        try:
            func = import_attr(import_name + name)
        except (ImportError, AttributeError):
            func = import_attr(name.lstrip(prefix))

        jinja_filters = func().items()
        for key, func in jinja_filters:
            app.jinja_env.filters[key] = func



from importlib import import_module

from flask import Blueprint, Flask

from ..helper import get_import_name, import_attr


def loader(app: Flask):
    """
    A function to load all of your API resources to flask based on the ``APIS`` configuration in config.py.
    """

    apis = app.config.get("APIS", [])
    import_name = get_import_name(app)
    api: Blueprint = import_attr(import_name + "apis.api")
    prefix = "apis."
    for name in apis:
        res = name
        if not name.startswith(prefix):
            res = prefix + res

        try:
            res = import_module(import_name + res)
        except ImportError:
            res = import_module(name.lstrip(prefix))

        endpoint = res.endpoint
        url_prefix = res.url_prefix
        routes = res.routes
        for detail in routes:
            route, view, methods = detail
            url = url_prefix + route
            e = endpoint + "_" + view.__name__
            api.add_url_rule(url, e, view_func=view, methods=methods)

    app.register_blueprint(api)



#!/usr/bin/env python

"""The setup script."""


from setuptools import find_packages, setup

from zemfrog import __author__, __email__, __version__

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

with open("requirements.txt") as req:
    requirements = req.read().splitlines()

with open("requirements-dev.txt") as req:
    test_requirements = req.read().splitlines()

setup_requirements = []


project_urls = {
    "Github": "https://github.com/zemfrog/zemfrog",
    "Issue Tracker": "https://github.com/zemfrog/zemfrog/issues",
    "Donation": "https://www.patreon.com/aprilahijriyan",
}

setup(
    author=__author__,
    author_email=__email__,
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Framework :: Flask",
        # "Framework :: Zemfrog",
        # "Framework :: Zemfrog :: " + __version__,
    ],
    description="Zemfrog is a simple framework based on flask for building a REST API quickly.",
    entry_points={"console_scripts": ["zemfrog=zemfrog.cli:main"]},
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="flask wsgi web zemfrog api",
    name="zemfrog",
    packages=find_packages(include=["zemfrog", "zemfrog.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/zemfrog/zemfrog",
    version=__version__,
    zip_safe=False,
    project_urls=project_urls,
)
